/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "local.h"

static void cream_view_setup_signals (CreamView *webview);

/* create a new CreamView structure */
CreamView *cream_view_new (void)
{
    CreamView *obj = g_malloc (sizeof (CreamView));

    g_return_val_if_fail (obj != NULL, NULL);

    obj->box = gtk_vbox_new (FALSE, 0);

    { /* viewport */
        GtkWidget *scroll_h = gtk_hscrollbar_new (NULL);
        GtkWidget *scroll_v = gtk_vscrollbar_new (NULL);

        obj->scroll.adjust_h = gtk_range_get_adjustment (GTK_RANGE (scroll_h));
        obj->scroll.adjust_v = gtk_range_get_adjustment (GTK_RANGE (scroll_v));

        obj->scroll.viewport = gtk_scrolled_window_new (obj->scroll.adjust_h, obj->scroll.adjust_v);
        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (obj->scroll.viewport),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    }

    { /* webkit */
        WebKitWebSettings *settings;
        WebKitWebInspector *inspector;

        obj->webview = webkit_web_view_new ();

        /* setting up global settings */
        settings = webkit_web_settings_new ();

        if (!global.javascript)
        {
            g_object_set (G_OBJECT (settings), "enable-scripts", FALSE, NULL);
        }

        if (0 != strcmp (global.encoding, "default"))
        {
            webkit_web_view_set_custom_encoding (WEBKIT_WEB_VIEW (obj->webview), global.encoding);
        }
        g_object_set (G_OBJECT (settings), "enable-developer-extras", TRUE, NULL);

        webkit_web_view_set_settings (WEBKIT_WEB_VIEW (obj->webview), settings);

        /* inspector */
        inspector = webkit_web_view_get_inspector (WEBKIT_WEB_VIEW (obj->webview));
        g_object_set (G_OBJECT (inspector), "javascript-profiling-enabled", TRUE, NULL);
        g_signal_connect (inspector, "inspect-web-view", (GCallback) create_inspect_win_cb, obj);
    }

    { /* statusbar */
        GtkWidget *statusbar = gtk_hbox_new (FALSE, 0);
        GdkColor bg;

        obj->statusbar.eventbox = gtk_event_box_new ();
        obj->statusbar.status_url = gtk_label_new (NULL);
        obj->statusbar.status_state = gtk_label_new (NULL);

        /* change statusbar color */
        gdk_color_parse ("#000000", &bg);
        gtk_widget_modify_bg (obj->statusbar.eventbox, GTK_STATE_NORMAL, &bg);

        /* align the URL and the state */
        gtk_misc_set_alignment (GTK_MISC (obj->statusbar.status_url), 0.0, 0.0);
        gtk_misc_set_alignment (GTK_MISC (obj->statusbar.status_state), 1.0, 0.0);

        gtk_label_set_selectable (GTK_LABEL (obj->statusbar.status_url), TRUE);

        gtk_box_pack_start (GTK_BOX (statusbar), obj->statusbar.status_url, TRUE, TRUE, 2);
        gtk_box_pack_start (GTK_BOX (statusbar), obj->statusbar.status_state, FALSE, FALSE, 2);

        gtk_container_add (GTK_CONTAINER (obj->statusbar.eventbox), statusbar);
    }

    { /* command line */
        PangoFontDescription *font;

        obj->command_line = gtk_entry_new ();

        /* change command line style */

        /* font */
        font = pango_font_description_from_string ("monospace normal 8");
        gtk_widget_modify_font (obj->command_line, font);
        pango_font_description_free (font);

        /* border */
        gtk_entry_set_inner_border (GTK_ENTRY (obj->command_line), NULL);
        gtk_entry_set_has_frame (GTK_ENTRY (obj->command_line), FALSE);
    }

    gtk_container_add (GTK_CONTAINER (obj->scroll.viewport), obj->webview);
    gtk_box_pack_start (GTK_BOX (obj->box), obj->scroll.viewport, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (obj->box), obj->statusbar.eventbox, FALSE, FALSE, 0);
    gtk_box_pack_end (GTK_BOX (obj->box), obj->command_line, FALSE, FALSE, 0);

    cream_view_setup_signals (obj);

    return obj;
}

/* connect widgets from 'obj' to their signal handler */
static void cream_view_setup_signals (CreamView *obj)
{
    g_object_connect ((GObject *) obj->webview,
        "signal::title-changed",                        (GCallback) webview_title_changed_cb,       obj,
        "signal::load-progress-changed",                (GCallback) webview_progress_changed_cb,    obj,
        "signal::load-committed",                       (GCallback) webview_load_committed_cb,      obj,
        "signal::load-finished",                        (GCallback) webview_load_finished_cb,       obj,
        "signal::navigation-requested",                 (GCallback) webview_navigation_cb,          obj,
        "signal::create-web-view",                      (GCallback) webview_new_window_cb,          obj,
        "signal::mime-type-policy-decision-requested",  (GCallback) webview_mimetype_cb,            obj,
        "signal::download-requested",                   (GCallback) webview_download_cb,            obj,
        "signal::hovering-over-link",                   (GCallback) webview_hoverlink_cb,           obj,
        "signal::console-message",                      (GCallback) webview_console_cb,             obj,
        "signal::populate-popup",                       (GCallback) webview_populate_menu_cb,       obj,
    NULL);

    g_object_connect ((GObject *) obj->scroll.adjust_v,
        "signal::value-changed",                        (GCallback) webview_scroll_cb,              obj,
    NULL);

    g_object_connect ((GObject *) obj->command_line,
        "signal::activate",                             (GCallback) input_activate_cb,              obj,
        "signal::key-press-event",                      (GCallback) inputbox_key_press_cb,          obj,
    NULL);
}

/* update the URL in the statusbar */
void cream_view_update_url (CreamView *obj, const gchar *url, gboolean link)
{
    gboolean ssl = g_str_has_prefix (url, "https://");
    GdkColor color;
    gboolean back = webkit_web_view_can_go_back (WEBKIT_WEB_VIEW (obj->webview));
    gboolean fwd = webkit_web_view_can_go_forward (WEBKIT_WEB_VIEW (obj->webview));

    /* hovering link ? */
    if (link)
    {
        gtk_label_set_markup ((GtkLabel*) obj->statusbar.status_url, g_markup_printf_escaped (
            "<span font=\"monospace bold 8\">Link: %s</span>", url
        ));
    }
    else if (back || fwd)
    {
        gtk_label_set_markup ((GtkLabel*) obj->statusbar.status_url, g_markup_printf_escaped (
            "<span font=\"monospace bold 8\">%s [%s]</span>", url,
            g_strconcat (back ? "+" : "", fwd ? "-" : "", NULL)
        ));
    }
    else
    {
        gtk_label_set_markup ((GtkLabel*) obj->statusbar.status_url, g_markup_printf_escaped (
            "<span font=\"monospace bold 8\">%s</span>", url
        ));
    }

    if (!link) /* if not, change statusbar color (ssl ?) */
    {
        gdk_color_parse (ssl ? "#B0FF00" : "#000000", &color);
        gtk_widget_modify_bg (obj->statusbar.eventbox, GTK_STATE_NORMAL, &color);

        gdk_color_parse (ssl ? "#000000" : "#FFFFFF", &color);
        gtk_widget_modify_fg (obj->statusbar.status_url, GTK_STATE_NORMAL, &color);
        gtk_widget_modify_fg (obj->statusbar.status_state, GTK_STATE_NORMAL, &color);
    }
}

/* update the state in the statusbar */
void cream_view_update_state (CreamView *obj, gint frac)
{
    gint max = gtk_adjustment_get_upper (obj->scroll.adjust_v) - gtk_adjustment_get_page_size (obj->scroll.adjust_v);
    gint val = (gint) (gtk_adjustment_get_value (obj->scroll.adjust_v) / max * 100);
    gint zoom = (gint) (webkit_web_view_get_zoom_level (WEBKIT_WEB_VIEW (obj->webview)) * 100.0);
    gchar *state, *markup, *tab;
    const gchar *encoding = webkit_web_view_get_encoding (WEBKIT_WEB_VIEW (obj->webview));

    if (max == 0) /* if we see all the page */
        state = g_strdup ("All");
    else if(val == 0) /* we are on the page's top */
        state = g_strdup ("Top");
    else if(val == 100) /* we are on the page's bottom */
        state = g_strdup ("Bot");
    else
        state = g_strdup_printf ("%d%%", val);

    /* [ current tab / number of tab ] */
    tab = g_strdup_printf ("[%d/%d]",
        gtk_notebook_page_num (GTK_NOTEBOOK (global.notebook), obj->box) + 1,
        gtk_notebook_get_n_pages (GTK_NOTEBOOK (global.notebook))
    );

    /* loading page */
    if (frac != 100)
    {
        markup = (gchar *) g_markup_printf_escaped ("<span font=\"monospace bold 8\">Progress: %02d%% - %s - %s - %d%% - %s</span>", frac, encoding, tab, zoom, state);
    }
    else /* page loaded */
    {
        markup = (gchar *) g_markup_printf_escaped ("<span font=\"monospace bold 8\">%s - %s - %d%% - %s</span>", encoding, tab, zoom, state);
    }

    gtk_label_set_markup (GTK_LABEL (obj->statusbar.status_state), markup);
}

/* load an FTP page */
void cream_view_load_ftp (CreamView *view, const gchar *uri)
{
    gchar *out = g_build_filename (g_get_user_cache_dir (), "cream", "ftp.out", NULL);
    gchar *ptn;

    netbuf *ftp_data;
    gchar *host = g_strdup (&uri[6]);
    gchar *username = "anonymous", *password = "";
    gchar *dir = "/";

    if ((ptn = strchr (host, '@')) != NULL)
    {
        gint i;

        username = g_strdup (host);
        for (i = 0; username[i] != ':'; ++i);
        username[i] = 0;

        password = g_strdup (&host[i + 1]);
        for (i = 0; password[i] != '@'; ++i);
        password[i] = 0;

        for (i = 1; host[i - 1] != '@'; ++i);
        host += i;
    }

    if ((ptn = strchr (host, '/')) != NULL)
    {
        dir = g_strdup (ptn);
        *ptn = 0;
    }

    cream_view_update_url (view, g_strconcat (_("Connecting to "), host, NULL), FALSE);
    FtpConnect (host, &ftp_data);
    FtpLogin (username, password, ftp_data);
    FtpNlst (out, dir, ftp_data);
    FtpQuit (ftp_data);

    /* print the file in the WebView */
    {
        gchar *content;
        gchar **tab;
        GError *err = NULL;

        if (!g_file_get_contents (out, &content, NULL, &err) && err != NULL)
        {
            g_printerr ("%s\n", err->message);
            return;
        }

        tab = g_strsplit (content, "\n", 0);

        if (tab != NULL)
        {
            GString *str = g_string_new (
                g_strdup_printf (FTP_VIEW_HTML_START, host, dir)
            );

            gint i;

            if (!g_str_equal (dir, "/"))
            {
                gchar *up = g_strdup (dir);

                ptn = strrchr (up, '/');
                *ptn = 0;

                /* if username:password is setted */
                if (strcmp (username, "anonymous"))
                {
                    g_string_append (str,
                        g_strdup_printf (FTP_VIEW_HTML_LINE,
                            g_strconcat ("ftp://",
                            username, ":",
                            password, "@",
                            host, up,
                            NULL),
                            "Parent directory"
                        )
                    );
                }
                else
                {
                    g_string_append (str,
                        g_strdup_printf (FTP_VIEW_HTML_LINE,
                            g_strconcat ("ftp://", host, up, NULL),
                            "Parent directory"
                        )
                    );
                }
            }

            for (i = 0; i < g_strv_length (tab); ++i)
            {
                gchar *ptn = strrchr (tab[i], '/');

                if (strlen (tab[i]) > 0)
                {
                    /* if username:password is setted */
                    if (strcmp (username, "anonymous"))
                    {
                        g_string_append (str,
                            g_strdup_printf (FTP_VIEW_HTML_LINE,
                                g_strconcat ("ftp://",
                                username, ":",
                                password, "@",
                                host, tab[i], NULL),
                                ptn != NULL ? ++ptn : tab[i]
                            )
                        );
                    }
                    else
                    {
                        g_string_append (str,
                            g_strdup_printf (FTP_VIEW_HTML_LINE,
                                g_strconcat ("ftp://", host, tab[i], NULL),
                                ptn != NULL ? ++ptn : tab[i]
                            )
                        );
                    }
                }
            }

            g_strfreev (tab);

            g_string_append (str, FTP_VIEW_HTML_END);

            webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (view->webview), g_string_free (str, FALSE), uri);
        }
    }
}

/* Execute JavaScript script */
gchar *jsapi_ref_to_string (JSContextRef context, JSValueRef ref)
{
    JSStringRef string_ref;
    gchar *string;
    size_t length;

    string_ref = JSValueToStringCopy (context, ref, NULL);
    length = JSStringGetMaximumUTF8CStringSize (string_ref);
    string = g_new (gchar, length);
    JSStringGetUTF8CString (string_ref, string, length);
    JSStringRelease (string_ref);

    return string;
}

void jsapi_evaluate_script (CreamView *obj, const gchar *script, gchar **value, gchar **message)
{
    WebKitWebFrame *frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (obj->webview));
    JSGlobalContextRef context = webkit_web_frame_get_global_context (frame);
    JSStringRef str;
    JSValueRef val, exception;

    str = JSStringCreateWithUTF8CString (script);
    val = JSEvaluateScript (context, str, JSContextGetGlobalObject (context), NULL, 0, &exception);
    JSStringRelease (str);

    if (!val)
    {
        *message = jsapi_ref_to_string (context, exception);
    }
    else
    {
        *value = jsapi_ref_to_string (context, val);
    }
}

void cream_view_script (CreamView *obj, const gchar *script)
{
    gchar *value = NULL, *message = NULL;

    g_return_if_fail (script != NULL);

    jsapi_evaluate_script (obj, script, &value, &message);

    if (message)
    {
        gtk_entry_set_text (GTK_ENTRY (obj->command_line), message);
        g_free (message);
    }

    if (value)
    {
        gtk_entry_set_text (GTK_ENTRY (obj->command_line), value);
        g_free (value);
    }
}

/* completion */
void cream_view_complete (CreamView *obj, CompletionArg arg)
{
    static GtkWidget *table, **widgets, *top_border;
    static char **suggestions, *prefix;
    static int n = 0, current = -1;

    extern Command cmdlist[];
    GdkColor color;
    GtkWidget *box, *row;
    GtkWidget *row_ev, *el, *el2;

    gboolean highlight = FALSE;
    char *str, *s, *p, *markup;
    int listlen, cmdlen, len;
    int i;

    str = (gchar *) gtk_entry_get_text (GTK_ENTRY (obj->command_line));
    len = strlen (str);

    if ((len == 0 || str[0] != ':') && arg != HideCompletion)
        return;

    if (prefix)
    {
        if (arg != HideCompletion && widgets && current != -1 && !strcmp (&str[1], suggestions[current]))
        {
            gdk_color_parse ("#FFFFFF", &color);
            gtk_widget_modify_bg (widgets[current], GTK_STATE_NORMAL, &color);
            current = (n + current + (arg == DirectionPrev ? -1 : 1)) % n;

            if ((arg == DirectionNext && current == 0)
                || (arg == DirectionPrev && current == n - 1))
            {
                current = -1;
            }
        }
        else
        {
            free (widgets);
            free (suggestions);
            free (prefix);

            gtk_widget_destroy (table);
            gtk_widget_destroy (top_border);

            table = NULL;
            top_border = NULL;
            widgets = NULL;
            suggestions = NULL;
            prefix = NULL;
            n = 0;
            current = -1;

            if (arg == HideCompletion) return;
        }
    }
    else if (arg == HideCompletion)
        return;

    if (!widgets)
    {
        prefix = g_strdup (str);

        for (listlen = 0; cmdlist[listlen].id != NULL; ++listlen);

        widgets = malloc (sizeof (GtkWidget *) * listlen);
        suggestions = malloc (sizeof (char *) * listlen);

        top_border = gtk_event_box_new ();
        gtk_widget_set_size_request (top_border, 0, 1);
        gdk_color_parse ("#000000", &color);
        gtk_widget_modify_bg (top_border, GTK_STATE_NORMAL, &color);
        table = gtk_event_box_new ();

        box = gtk_vbox_new (FALSE, 0);
        highlight = len > 1;

        for (i = 0; i < listlen; ++i)
        {
            cmdlen = strlen (cmdlist[i].cmd);

            if (!highlight || (len - 1 <= cmdlen && !strncmp (&str[1], cmdlist[i].cmd, len - 1)))
            {
                p = s = malloc (sizeof (char *) * (highlight ? sizeof ("<b>") + sizeof ("</b>") - 1 : 1) + cmdlen);

                if (highlight)
                {
                    memcpy (p, "<b>", sizeof ("<b>") - 1);
                    memcpy ((p += sizeof ("<b>") - 1), &str[1], len - 1);
                    memcpy ((p += len - 1), "</b>", sizeof ("</b>") - 1);
                    p += sizeof ("</b>") - 1;
                }

                memcpy (p, &cmdlist[i].cmd[len - 1], cmdlen - len + 2);

                row = gtk_hbox_new (FALSE, 0);
                row_ev = gtk_event_box_new ();
                gdk_color_parse ("#FFFFFF", &color);
                gtk_widget_modify_bg (row_ev, GTK_STATE_NORMAL, &color);

                el = gtk_label_new (NULL);
                markup = g_strdup_printf ("<span font=\"monospace normal 8\" color=\"#000000\">%s</span>", s);
                free (s);

                gtk_label_set_markup (GTK_LABEL (el), markup);
                g_free (markup);

                el2 = gtk_label_new (NULL);
                markup = g_strdup_printf ("<span font=\"monospace normal 8\" color=\"#005500\">%s</span>", cmdlist[i].desc);
                gtk_label_set_markup (GTK_LABEL (el2), markup);
                g_free (markup);

                gtk_misc_set_alignment (GTK_MISC (el), 0, 0);
                gtk_box_pack_start (GTK_BOX (row), el, TRUE, TRUE, 2);
                gtk_box_pack_end (GTK_BOX (row), el2, FALSE, FALSE, 2);
                gtk_container_add (GTK_CONTAINER (row_ev), row);
                gtk_box_pack_start (GTK_BOX (box), row_ev, FALSE, FALSE, 0);

                suggestions[n] = cmdlist[i].cmd;
                widgets[n++] = row_ev;
            }
        }

        widgets = realloc (widgets, sizeof (GtkWidget *) * n);
        suggestions = realloc (suggestions, sizeof (char *) * n);

        if (!n)
        {
            gdk_color_parse ("#FFFFFF", &color);
            gtk_widget_modify_bg (table, GTK_STATE_NORMAL, &color);

            el = gtk_label_new (NULL);
            gtk_misc_set_alignment (GTK_MISC (el), 0, 0);
            gtk_label_set_markup (GTK_LABEL (el), "<span font=\"monospace bold 8\" color=\"#FF00FF\">No completions</span>");
            gtk_box_pack_start (GTK_BOX (box), el, FALSE, FALSE, 0);
        }

        gtk_box_pack_start (GTK_BOX (obj->box), top_border, FALSE, FALSE, 0);
        gtk_container_add (GTK_CONTAINER (table), box);
        gtk_box_pack_start (GTK_BOX (obj->box), table, FALSE, FALSE, 0);
        gtk_widget_show_all (global.win);

        if (!n) return;
        current = arg == DirectionPrev ? n - 1 : 0;
    }

    if (current != -1)
    {
        gdk_color_parse ("#FFF000", &color);
        gtk_widget_modify_bg (widgets[current], GTK_STATE_NORMAL, &color);
        echo (obj, ":%s", suggestions[current]);
    }
    else
    {
        echo (obj, "%s", prefix);
    }

    gtk_widget_grab_focus (obj->command_line);
    gtk_editable_set_position (GTK_EDITABLE (obj->command_line), -1);
}

/* Print in the Inputbox */
void echo (CreamView *obj, const gchar *fmt, ...)
{
    va_list args;
    gchar *str;

    va_start (args, fmt);
    str = g_strdup_vprintf (fmt, args);
    va_end (args);

    gtk_entry_set_text (GTK_ENTRY (obj->command_line), str);
    g_free (str);

    gtk_widget_grab_focus (obj->webview);
}
