/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "local.h"

Command cmdlist[] =
{
    /* identifier   command         args    function        description */
    { "open",       "o",            -1,     cmd_open,       "Open URL" },
    { "tabopen",    "t",            -1,     cmd_tabopen,    "Open URL in a new window" },
    { "close",      "c",            0,      cmd_close,      "Close current window" },
    { "quit",       "q",            0,      cmd_quit,       "Exit program" },
    { "search",     "s",            -1,     cmd_search,     "Search text in the current page" },
    { "yank",       "y",            0,      cmd_yank,       "Copy current URL in the clipboard" },
    { "paste",      "p",            -1,     cmd_paste,      "Open the yanked URL" },
    { "trash",      "trash",        0,      cmd_trash,      "Open the last closed page" },
    { "history",    "history",      0,      cmd_hist,       "Show navigation's history" },
    { "bookmarks",  "bookmarks",    0,      cmd_book,       "Show bookmarks" },
    { NULL, NULL, 0, NULL, NULL }
};

static CreamView *current = NULL;
GSList *cmd_history = NULL;
int current_item = 0;

/* get a command with its identifier */
gchar *get_command (const gchar *identifier)
{
    gint i;

    for (i = 0; cmdlist[i].id != NULL; ++i)
    {
        if (!strcmp (cmdlist[i].id, identifier))
        {
            return g_strconcat (cmdlist[i].cmd, " ", NULL);
        }
    }

    return "";
}

/* set description */
void set_desc (const gchar *identifier, const gchar *desc)
{
    gint i;
    for (i = 0; cmdlist[i].id != NULL; ++i)
    {
        if (!strcmp (cmdlist[i].id, identifier))
        {
            cmdlist[i].desc = g_strdup (desc);
        }
    }
}

void add_to_history (const gchar *command)
{
    cmd_history = g_slist_append (cmd_history, g_string_new (command));
    current_item = g_slist_length (cmd_history);
}

/* New command */
void input_activate_cb (GtkWidget *widget, gpointer data)
{
    const gchar *text = gtk_entry_get_text (GTK_ENTRY (widget));
    current = (CreamView *) data;

    g_return_if_fail (text != NULL);
    if (strlen (text) == 0) return;

    webkit_web_view_unmark_text_matches (WEBKIT_WEB_VIEW (current->webview));
    cream_view_complete (current, HideCompletion);

    getcmd (widget, text);
}

/* get the command */
void getcmd (GtkWidget *entry, const gchar *text)
{
    gchar **argv;
    gint i;

    if (text[0] != ':' || !(strlen (text) > 1))
    {
        echo (current, _("Not a browser command : ':'"));
        return;
    }

    argv = g_strsplit (&text[1], " ", 0);
    if (argv == NULL) return;

    /* search in the list */
    for (i = 0;; ++i)
    {
        if (cmdlist[i].id != NULL)
        {
            if (!strcmp (argv[0], cmdlist[i].cmd))
            {
                if (cmdlist[i].n_args >= 0)
                {
                    if (g_strv_length (argv) != (cmdlist[i].n_args + 1))
                    {
                        echo (current, _("Arguments error : %s"), cmdlist[i].id);
                        break;
                    }
                }
                else
                {
                    if (g_strv_length (argv) < ABS (cmdlist[i].n_args) + 1)
                    {
                        echo (current, _("Arguments error : %s"), cmdlist[i].id);
                        break;
                    }
                }

                add_to_history (text);
                echo (current, "");
                cmdlist[i].func (g_strv_length (argv), argv);

                break;
            }
        }
        else /* command error */
        {
            echo (current, _("Not a browser command : '%s'"), argv[0]);
            break;
        }
    }

    g_strfreev (argv);
}

void cmd_open (gint argc, gchar **argv)
{
    if (argc == 2)
    {
        gchar *url = (argv[1] != NULL ? argv[1] : "about:blank");

        /* no protocole specified ? */
        if (!g_str_has_prefix (url, "http://")
            && !g_str_has_prefix (url, "https://")
            && !g_str_has_prefix (url, "file://")
            && !g_str_has_prefix (url, "ftp://"))
        {
            if (strchr (url, '.') == NULL) /* no extension (.com/.net/etc...) */
            {
                webkit_web_view_open (WEBKIT_WEB_VIEW (current->webview),
                    g_strconcat ("http://www.google.com/search?q=", url, NULL)
                );
            }
            else /* default protocole : HTTP */
            {
                webkit_web_view_open (WEBKIT_WEB_VIEW (current->webview),
                    g_strconcat ("http://", url, NULL)
                );
            }
        }
        else
        {
            webkit_web_view_open (WEBKIT_WEB_VIEW (current->webview), url);
        }
    }
    else if (argc > 2) /* more arguments, search on google */
    {
        gchar **tmp = &argv[1];

        webkit_web_view_open (WEBKIT_WEB_VIEW (current->webview),
            g_strconcat ("http://www.google.com/search?q=", g_strjoinv ("+", tmp), NULL)
        );
    }
}

void cmd_tabopen (gint argc, gchar **argv)
{
    if (argc == 2)
    {
        gchar *url = (argv[1] != NULL ? argv[1] : "about:blank");

        /* no protocole specified ? */
        if (!g_str_has_prefix (url, "http://")
            && !g_str_has_prefix (url, "https://")
            && !g_str_has_prefix (url, "file://"))
        {
            if (strchr (url, '.') == NULL) /* no extension (.com/.net/etc...) */
            {
                add_cream_view_to_notebook (
                    g_strconcat ("http://www.google.com/search?q=", url, NULL)
                );
            }
            else /* default protocole : HTTP */
            {
                add_cream_view_to_notebook (
                    g_strconcat ("http://", url, NULL)
                );
            }
        }
        else
        {
            add_cream_view_to_notebook (url);
        }
    }
    else if (argc > 2) /* more arguments, search on google */
    {
        gchar **tmp = &argv[1];

        add_cream_view_to_notebook (
            g_strconcat ("http://www.google.com/search?q=", g_strjoinv ("+", tmp), NULL)
        );
    }

}

void cmd_close (gint argc, gchar **argv)
{
    global.trash = g_slist_append (global.trash, g_string_new (
        webkit_web_view_get_uri (WEBKIT_WEB_VIEW (current->webview))
    ));

    gtk_notebook_remove_page (GTK_NOTEBOOK (global.notebook),
        gtk_notebook_current_page (GTK_NOTEBOOK (global.notebook))
    );
}

void cmd_quit (gint argc, gchar **argv)
{
    quit (NULL, NULL);
}

void cmd_search (gint argc, gchar **argv)
{
    gint i;

    for (i = 1; i < argc; ++i)
        webkit_web_view_mark_text_matches (WEBKIT_WEB_VIEW (current->webview), argv[i], TRUE, 0);

    webkit_web_view_set_highlight_text_matches (WEBKIT_WEB_VIEW (current->webview), TRUE);
}

void cmd_yank (gint argc, gchar **argv)
{
    const gchar *uri = webkit_web_view_get_uri (WEBKIT_WEB_VIEW (current->webview));
    echo (current, "Yanked %s", uri);
    gtk_clipboard_set_text (global.clipboard, uri, -1);
}

void cmd_paste (gint argc, gchar **argv)
{
    gchar *uri = gtk_clipboard_wait_for_text (global.clipboard);

    if (argc == 2 && !strcmp (argv[1], "tab"))
        add_cream_view_to_notebook (uri);
    else
        webkit_web_view_open (WEBKIT_WEB_VIEW (current->webview), uri);
}

void cmd_trash (gint argc, gchar **argv)
{
    GSList *tmp = g_slist_last (global.trash);

    if (tmp != NULL)
    {
        GString *str = (GString *) g_slist_nth_data (tmp, 0);
        add_cream_view_to_notebook (str->str);
        global.trash = g_slist_delete_link (global.trash, tmp);
    }
    else
    {
        echo (current, _("Trash is empty"));
    }
}

void cmd_hist (gint argc, gchar **argv)
{
    show_history ();
}

void cmd_book (gint argc, gchar **argv)
{
    show_bookmarks ();
}
