/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "local.h"

static GSList *cream_history = NULL;

void history_read_file (void)
{
    gchar **lines = NULL;
    GError *error = NULL;
    gchar *contents = NULL;
    gint i;

    if (!g_file_get_contents (HISTORY_FILE, &contents, NULL, &error))
    {
        if (error != NULL)
        {
            fprintf (stderr, _("Error while reading %s : %s\n"), HISTORY_FILE, error->message);
            g_error_free (error);
        }
        return;
    }

    lines = g_strsplit (contents, "\n", 0);
    g_free (contents);

    if (lines == NULL) return;

    for (i = 0; lines[i] != NULL; ++i)
    {
        if (strlen (lines[i]) > 0)
        {
            HistoryItem *tmp = malloc (sizeof (HistoryItem));
            int x;

            for (x = 0; lines[i][x] != '.'; ++x);
            lines[i][x] = 0;

            tmp->timestamp = (int) strtol (lines[i], NULL, 10);

            lines[i] += (x + 1);
            for (x = 0; lines[i][x] != ' '; ++x);
            lines[i][x] = 0;

            tmp->uri = g_strdup (lines[i]);

            lines[i] += (x + 1);

            tmp->title = g_strdup (lines[i]);

            cream_history = g_slist_append (cream_history, tmp);
        }
    }
}

void history_write_file (void)
{
    GString *content = g_string_new ("");
    GError *error = NULL;
    int i;

    for (i = 0; i < g_slist_length (cream_history); ++i)
    {
        HistoryItem *tmp = (HistoryItem *) g_slist_nth_data (cream_history, i);

        g_string_append_printf (content, "%d.%s %s\n", (int) tmp->timestamp, tmp->uri, tmp->title);
    }

    if (!g_file_set_contents (HISTORY_FILE, content->str, strlen (content->str), &error))
    {
        if (error != NULL)
        {
            fprintf (stderr, _("Error while writing %s : %s\n"), HISTORY_FILE, error->message);
            g_error_free (error);
        }
    }

    g_string_free (content, TRUE);
    g_slist_free (cream_history);
}

void history_add_item (const gchar *uri, const gchar *title)
{
    gboolean success = FALSE;
    int i;

    for (i = 0; i < g_slist_length (cream_history); ++i)
    {
        HistoryItem *tmp = (HistoryItem *) g_slist_nth_data (cream_history, i);

        if (!strcmp (tmp->uri, uri))
        {
            tmp->timestamp = time (NULL);
            g_free (tmp->title);
            tmp->title = g_strdup (title);

            success = TRUE;
            break;
        }
    }

    if (!success)
    {
        HistoryItem *tmp = malloc (sizeof (HistoryItem));
        tmp->timestamp = time (NULL);
        tmp->uri = g_strdup (uri);
        tmp->title = g_strdup (title);

        cream_history = g_slist_append (cream_history, tmp);
    }
}

void history_delete_item (const gchar *uri, const gchar *title)
{
    int i;

    for (i = 0; i < g_slist_length (cream_history); ++i)
    {
        HistoryItem *tmp = (HistoryItem *) g_slist_nth_data (cream_history, i);

        if (!strcmp (tmp->uri, uri) && !strcmp (tmp->title, title))
        {
            cream_history = g_slist_remove (cream_history, tmp);
            g_free (tmp->uri);
            g_free (tmp->title);
            free (tmp);
            return;
        }
    }
}

static void renderer_toggled (  GtkCellRendererToggle *renderer,
                                gchar *path,
                                gpointer data)
{
    GtkListStore *store = (GtkListStore *) data;
    GtkTreeIter iter;
    gboolean value = FALSE;

    gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, path);

    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 3, &value, -1);
    gtk_list_store_set (store, &iter, 3, !value, -1);
}

void show_history (void)
{
    GtkWidget *window;
    gint i;

    GtkWidget *scroll;
    GtkWidget *listview;
    GtkListStore *store;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cellrenderer;

    if (!g_slist_length (cream_history))
    {
        echo (notebook_get_creamview (-1), _("History is empty"));
        return;
    }

    window = gtk_dialog_new_with_buttons (_("History"),
        GTK_WINDOW (global.win), 0,
        GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
        GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
        NULL);
    gtk_window_set_default_size (GTK_WINDOW (window), 640, 480);

    /* init data list */
    store = gtk_list_store_new (4,
        G_TYPE_STRING,  /* timestamp */
        G_TYPE_STRING,  /* URL */
        G_TYPE_STRING,  /* Title */
        G_TYPE_BOOLEAN  /* delete ? */
    );

    /* set datas */
    for (i = 0; i < g_slist_length (cream_history); ++i)
    {
        HistoryItem *tmp = (HistoryItem *) g_slist_nth_data (cream_history, i);
        GtkTreeIter iter;
        gchar *timestamp = ctime (&(tmp->timestamp));

        char *ptn = strchr (timestamp, '\n');
        if (ptn != NULL) *ptn = 0;

        gtk_list_store_append (GTK_LIST_STORE (store), &iter);
        gtk_list_store_set (GTK_LIST_STORE (store), &iter,
            0, timestamp,
            1, tmp->uri,
            2, tmp->title,
            3, FALSE,
            -1);
    }

    /* create treeview */
    listview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));

    /* first column */
    cellrenderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes (_("Date"),
        cellrenderer,
        "text", 0,
        NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (listview), column);

    /* second column */
    cellrenderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes (_("Link"),
        cellrenderer,
        "text", 1,
        NULL);
    gtk_tree_view_column_set_max_width (column, 200);
    gtk_tree_view_append_column (GTK_TREE_VIEW (listview), column);

    /* third column */
    cellrenderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes (_("Title"),
        cellrenderer,
        "text", 2,
        NULL);
    gtk_tree_view_column_set_max_width (column, 200);
    gtk_tree_view_append_column (GTK_TREE_VIEW (listview), column);

    /* last column */
    cellrenderer = gtk_cell_renderer_toggle_new ();
    gtk_cell_renderer_toggle_set_active (GTK_CELL_RENDERER_TOGGLE (cellrenderer), TRUE);
    g_object_set (G_OBJECT (cellrenderer), "activatable", TRUE, NULL);
    g_signal_connect (G_OBJECT (cellrenderer), "toggled", G_CALLBACK (renderer_toggled), store);

    column = gtk_tree_view_column_new_with_attributes (_("Delete"),
        cellrenderer,
        "active", 3,
        NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (listview), column);

    /* add to dialogbox */
    scroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    gtk_container_add (GTK_CONTAINER (scroll), listview);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), scroll, TRUE, TRUE, 0);

    gtk_widget_show_all (GTK_DIALOG (window)->vbox);

    if (gtk_dialog_run (GTK_DIALOG (window)) == GTK_RESPONSE_APPLY)
    {
        GtkTreeIter iter;

        gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);

        do
        {
            gchar *uri = NULL, *title = NULL;
            gboolean value = FALSE;

            gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
                1, &uri,
                2, &title,
                3, &value,
                -1);

            if (value)
            {
                history_delete_item (uri, title);
            }
        } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter));
    }

    gtk_widget_destroy (window);
}
