/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "local.h"

CreamGlobal global;

/* Create a tab-label for Gtk.NoteBook (favicon + page title + close button) */
GtkWidget *notebook_create_tab (CreamView *child)
{
    GtkWidget *hbox, *label, *button;

    gchar *txt = g_strdup (webkit_web_view_get_title (WEBKIT_WEB_VIEW (child->webview)));
    GtkRcStyle *rcstyle;
    GtkWidget *image;

    hbox = gtk_hbox_new (FALSE, 0);

    gtk_box_pack_start (GTK_BOX (hbox),
        favicon_load_from_uri (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (child->webview))),
        FALSE, FALSE, 0);

    /* truncate message */
    if (txt == NULL)
        txt = g_strdup ("about:blank");

    if (strlen (txt) > 25)
    {
        txt[24] = 0;
        txt = g_strdup_printf ("%s...", txt);
    }

    label = gtk_label_new (txt);

    /* button */
    image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
    gtk_misc_set_alignment (GTK_MISC (image), 0.0, 0.0);

    rcstyle = gtk_rc_style_new ();
    rcstyle->xthickness = rcstyle->ythickness = 0;

    button = gtk_button_new ();
    gtk_button_set_image (GTK_BUTTON (button), image);
    gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);

    gtk_widget_modify_style (button, rcstyle);
    g_object_unref (rcstyle);

    g_signal_connect (button, "clicked", (GCallback) notebook_close_page_cb, child);

    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 5);
    gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);

    gtk_widget_show_all (hbox);

    return hbox;
}

/* Open an URL in a new tab */
void add_cream_view_to_notebook (const gchar *url)
{
    CreamView *cream = cream_view_new ();
    webkit_web_view_open (WEBKIT_WEB_VIEW (cream->webview), url);

    gtk_notebook_append_page (GTK_NOTEBOOK (global.notebook), cream->box,
        notebook_create_tab (cream)
    );

    gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (global.notebook), cream->box, TRUE);
}

/* callback: end of program */
void quit (GtkWidget *emitter, gpointer data)
{
    FILE *f = fopen (SESSION_FILE, "w");

    if (f != NULL)
    { /* save session */
        gint n = gtk_notebook_get_n_pages (GTK_NOTEBOOK (global.notebook));
        gint i;

        for (i = 0; i < n; ++i)
        {
            WebKitWebView *webview = WEBKIT_WEB_VIEW (notebook_get_creamview (i)->webview);

            const gchar *url = webkit_web_view_get_uri (webview);

            if (url != NULL
                && !g_str_has_prefix (url, "about:")
                && !g_str_has_prefix (url, "error:"))
            {
                fprintf (f, "%s\n", url);
            }
        }

        fclose (f);
    }

    /* save history/bookmarks */
    history_write_file ();
    bookmarks_write_file ();

    /* save cookies */
    soup_cookie_jar_save (global.cookies);

    /* free trash stack */
    g_slist_free (global.trash);

    /* TODO: Run this function only if the main loop is running */
    gtk_main_quit ();
    exit (global.code);
}

void seg_fault_exit (int signum)
{
    fprintf (stderr, "Cream: Segmentation Fault, exiting...\n");
    global.code = EXIT_FAILURE;
    quit (NULL, NULL);
}

int main (int argc, char *argv[])
{
    gchar *content;
    GError *error = NULL;
    GdkGeometry hints = { 1, 1 }; /* GDK_HINT_MIN_SIZE for global.win */

    GOptionContext *ctx;
    gchar *url = NULL;
    gchar *config_file = CONFIG_FILE;
    GOptionEntry entries[] =
    {
        { "config", 'c', 0, G_OPTION_ARG_FILENAME, &config_file, _("Load an alternative configuration file") },
        { "open", 'o', 0, G_OPTION_ARG_STRING, &url, _("Open URL") },
        { NULL }
    };

    global.code = EXIT_SUCCESS;

    /* set locale */
    setlocale (LC_ALL, "");
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);

    signal (SIGSEGV, seg_fault_exit);

    /* parse command line */
    ctx = g_option_context_new ("");
    g_option_context_add_main_entries (ctx, entries, PACKAGE);
    g_option_context_add_group (ctx, gtk_get_option_group (TRUE));

    if (!g_option_context_parse (ctx, &argc, &argv, &error) && error != NULL)
    {
        fprintf (stderr, _("Option parsing failed : %s\n"), error->message);
        g_error_free (error);
        error = NULL;
    }

    /* init GTK/Glib-threading */
    gtk_init (&argc, &argv);

    if (!g_thread_supported ())
        g_thread_init (NULL);

    /* load configuration */
    global.homepage = "http://www.google.fr";
    global.encoding = "default";
    global.javascript = TRUE;
    global.trash = NULL;

    if (-1 == config_load (config_file))
        fprintf (stderr, _("Error: Can't load configuration file : %s\n"), CONFIG_FILE);

    { /* setting up modkeys */
        extern Keybinding shortkeys[];
        unsigned int i, len;
        char *ptr;

        for (len = 0; shortkeys[len].id != NULL; ++len);

        global.modkeys = calloc (len + 1, sizeof (char));
        ptr = global.modkeys;

        for (i = 0; i < len; ++i)
        {
            if (shortkeys[i].modkey && !strchr (global.modkeys, shortkeys[i].modkey))
            {
                *(ptr++) = shortkeys[i].modkey;
            }
        }

        global.modkeys = realloc (global.modkeys, &ptr[0] - &global.modkeys[0] + 1);
    }

    /* setting up commands' descriptions */
    set_desc ("open",       _("Open URL"));
    set_desc ("tabopen",    _("Open URL in a new window"));
    set_desc ("close",      _("Close current window"));
    set_desc ("quit",       _("Exit program"));
    set_desc ("search",     _("Search text in the current page"));
    set_desc ("yank",       _("Copy current URL in the clipboard"));
    set_desc ("paste",      _("Open the yanked URL"));
    set_desc ("trash",      _("Open the last closed page"));
    set_desc ("history",    _("Show navigation's history"));
    set_desc ("bookmarks",  _("Show bookmarks"));

    /* setting up GUI */
    global.clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);

    /* create window */
    global.win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (global.win), "Cream " VERSION);
    gtk_window_set_default_size (GTK_WINDOW (global.win), 800, 600);
    gtk_window_set_geometry_hints (GTK_WINDOW (global.win), NULL, &hints, GDK_HINT_MIN_SIZE);

    /* create notebook */
    global.notebook = gtk_notebook_new ();
    gtk_container_add (GTK_CONTAINER (global.win), global.notebook);

    /* open URL given in command line */
    if (url != NULL)
        add_cream_view_to_notebook (url);

    /* re-open old session */
    g_file_get_contents (SESSION_FILE, &content, NULL, &error);

    if (error != NULL)
    {
        g_printerr ("%s\n", error->message);
        g_error_free (error);

        add_cream_view_to_notebook (global.homepage);
    }
    else
    {
        if (strlen (content) > 0)
        {
            gchar **tab = g_strsplit (content, "\n", 0);

            if (tab != NULL)
            {
                int i;

                for (i = 0; tab[i] != NULL; ++i)
                    if (strlen (tab[i]) > 0)
                        add_cream_view_to_notebook (tab[i]);

                g_strfreev (tab);
            }
            else
                add_cream_view_to_notebook (global.homepage);
        }
        else
            add_cream_view_to_notebook (global.homepage);

        g_free (content);
    }

    /* read history/bookmarks */
    history_read_file ();
    bookmarks_read_file ();

    /* restore cookies */
    global.cookies = soup_cookie_jar_text_new (COOKIE_FILE, FALSE);
    soup_session_add_feature (webkit_get_default_session (), SOUP_SESSION_FEATURE (global.cookies));

    /* setting up signals */
    g_signal_connect (global.win, "destroy", (GCallback) quit, NULL);

    g_signal_connect_after (global.win,
        "key-press-event", (GCallback)  window_key_press, NULL
    );

    g_object_connect ((GObject *) global.notebook,
        "signal::page-added",       (GCallback) notebook_page_added_cb,     NULL,
        "signal::page-removed",     (GCallback) notebook_page_removed_cb,   NULL,
        "signal::page-reordered",   (GCallback) notebook_page_reordered_cb, NULL,
        "signal::switch-page",      (GCallback) notebook_switch_page_cb,    NULL,
    NULL);

    /* show GUI */
    gtk_widget_show_all (global.win);
    gtk_main ();
    return 0;
}

/* Restore a CreamView structure
 *
 * i = -1 : current page
 * else page number 'i'
 *
 * gtk.notebook
 *  +- page
 *          +- vertical box
 *              +- scrolled window
 *                  |- webkit widget
 *                  |- gtk.vadjustment
 *                  |- gtk.hadjustment
 *              +- statusbar
 *                  |- url
 *                  |- state
 *              |- command line
 */
CreamView *notebook_get_creamview (gint i)
{
    gint x = (i < 0 ? gtk_notebook_get_current_page (GTK_NOTEBOOK (global.notebook)) : i);
    GList *list, *list2;
    CreamView *obj;

    obj = malloc (sizeof (CreamView));

    g_return_val_if_fail (obj != NULL, NULL);

    obj->box = gtk_notebook_get_nth_page (GTK_NOTEBOOK (global.notebook), x);

    list = gtk_container_get_children (GTK_CONTAINER (obj->box));

    /* scrolled window */
    obj->scroll.viewport = g_list_nth_data (list, 0);
    obj->scroll.adjust_h = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (obj->scroll.viewport));
    obj->scroll.adjust_v = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (obj->scroll.viewport));

    /* webview */
    list2 = gtk_container_get_children (GTK_CONTAINER (obj->scroll.viewport));
    obj->webview = g_list_nth_data (list2, 0);

    /* statusbar */
    obj->statusbar.eventbox = g_list_nth_data (list, 1);

    list2 = gtk_container_get_children (GTK_CONTAINER (obj->statusbar.eventbox));
    list2 = gtk_container_get_children (GTK_CONTAINER (g_list_nth_data (list2, 0)));
    obj->statusbar.status_url = g_list_nth_data (list2, 0);
    obj->statusbar.status_state = g_list_nth_data (list2, 1);

    /* command line */
    obj->command_line = g_list_nth_data (list, 2);

    return obj;
}

gchar *str_replace (const gchar *string, const gchar *substr, const gchar *replace)
{
    gchar **tmp;
    gchar *ret;

    tmp = g_strsplit (string, substr, -1);
    ret = g_strjoinv (replace, tmp);
    g_strfreev (tmp);

    return ret;
}
